/*
   Feet of Fury

   demo/demo.cc

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#include "global.h"

/*

 PUBLIC SOURCE NOTE:

 This file is no longer used. It's one of the apocryphal pieces of FoF
 that never made it into the final cut due to time.

*/

/*

 I'm abusing poor FoF for all its worth here to test out some ideas I had
 with regard to scripting in an event-driven environment without having
 to have event-driven scripts. This module is a pretty big departure from
 the code design in the rest of FoF, but what can I say.. ^_^;

 The basic idea here is that the entire system is state based and re-entrant.
 The main loop can easily end up calling itself via Lua scripting calls,
 provided it's in a "state safe" condition, meaning that it's not in
 the middle of something that shouldn't be interrupted (like refreshing
 the screen).

 The downside to this is that an error in a Lua script can kill the whole
 system, but I suppose that's possible no matter how you work it. The big
 advantage is that you no longer have to think in terms of state after
 the main piece is written -- the scripting code can work in a standard,
 imperative manner.

 Later on these will probably move into the character animation in the
 main game as well as into the RPG toolkit I'm working on.

*/

// Scrolly bg
static ScrollyBG * scrolly;

// Font helper
static FontHelper * fnt;

// Player
static Player * pv;
static GameplayState * gps;
static bool pv_visible;
static float pv_alpha;

// Frame counter
static int frame;

// Banner location/size
static float banner_x, banner_y, banner_ps, banner_alpha;
static const char * banner_text = "How to Play";

// Text to display (if any)
static char * text[16] = { NULL };
static int text_cnt = 0;
static bool text_fading;
static float text_w, text_h;
static float text_alpha;

// Texture set
static Texture ** txrs = NULL;
static int txrs_cnt = 0;
static int txrs_current = -1;

// WaveTexture object for transitions
static WaveTexture * wtold, * wtnew;

// Alpha fade value for birth/death
static float alpha = 0.0f;

/////////////////////////////////////////////////////////////////////////////
// Per-frame stuff

static bool check_control() {
	maple_device_t	*addr;
	cont_state_t	*st;
	int		i;

	i = 0;
	while ( (addr = maple_enum_type(i++, MAPLE_FUNC_CONTROLLER)) ) {
		st = (cont_state_t *)maple_dev_status(addr);
#ifdef SS_ENABLE
		if (st->ltrig > 0) {
			printf("writing screen shot...\n");
			vid_screen_shot("/pc/tmp/ss.ppm");
			printf("done!\n");
		}
#endif
	                                                        
		if (st->buttons & (CONT_START | CONT_B))
			return true;
	}
	
	return false;
}

static void do_frame() {
	glKosBeginFrame();

	scrolly->set_alpha(alpha);
	scrolly->draw(GL_LIST_OPAQUE_POLY);
	scrolly->next_frame();

	glKosFinishList();

	if (banner_ps > 100.0f)
		fnt->draw_centered(banner_x, banner_y, 128.0f, banner_ps, banner_text, alpha);
	else
		fnt->smear_draw_centered(banner_x, banner_y, 128.0f, banner_ps, banner_text, alpha * banner_alpha);

	if (text_cnt > 0) {
		// Draw a bounding box
		glLoadIdentity();
		glDisable(GL_TEXTURE_2D);
		glColor4f(0.1f, 0.1f, 0.5f, 0.8f * alpha * text_alpha);
		float tw = text_w/2, th = text_h/2;
		glBegin(GL_QUADS);
			glVertex3f(200.0f-tw, 392.0f-th, 80.0f);
			glVertex3f(200.0f-tw, 392.0f+th, 80.0f);
			glVertex3f(200.0f+tw, 392.0f+th, 80.0f);
			glVertex3f(200.0f+tw, 392.0f-th, 80.0f);
		glEnd();

		glColor4f(0.2f, 0.2f, 0.3f, 0.7f * alpha * text_alpha);
		glBegin(GL_QUADS);
			glVertex3f(204.0f-tw, 396.0f-th, 79.0f);
			glVertex3f(204.0f-tw, 396.0f+th, 79.0f);
			glVertex3f(204.0f+tw, 396.0f+th, 79.0f);
			glVertex3f(204.0f+tw, 396.0f-th, 79.0f);
		glEnd();
		glEnable(GL_TEXTURE_2D);

		// Draw the actual chars
		float y = 408.0f - text_h/2;
		for (int i=0; i<text_cnt; i++) {
			fnt->draw_centered(200.0f, y, 128.0f, 16.0f, text[i], alpha * text_alpha);
			y += 16.0f;
		}

		if (!text_fading) {
			if (text_alpha < 1.0f)
				text_alpha += 0.1f;
			else if (text_alpha > 1.0f)
				text_alpha = 1.0f;
		}
	}

	if (pv_visible) {
		gps->alpha = pv_alpha * alpha;
		pv->visual_per_frame();
		gps->sync_time();
		gps->frame++;
	}

	if (wtold || wtnew) {
		if (wtold) {
			wtold->draw(GL_LIST_TRANS_POLY);
			wtold->next_frame();

			wtold->m_mag += 10.0f/80.0f;
			wtold->m_alpha -= 1.0f/80.0f;
			if (wtold->m_alpha < 0.0f) {
				delete wtold;
				wtold = NULL;
			}
		}
		if (wtnew) {
			wtnew->draw(GL_LIST_TRANS_POLY);
			wtnew->next_frame();

			if (wtnew->m_alpha >= 1.0f || wtnew->m_mag <= 0.0f) {
				delete wtnew;
				wtnew = NULL;
			}
			wtnew->m_mag -= 10.0f/80.0f;
			wtnew->m_alpha += 1.0f/80.0f;
			if (wtold->m_mag < 0.0f)
				wtold->m_mag = 0.0f;
			if (wtold->m_alpha > 1.0f)
				wtold->m_alpha = 1.0f;
		}
	} else if (txrs_current != -1) {
		glLoadIdentity();
		glEnable(GL_TEXTURE_2D);
		glColor4f(1.0f, 1.0f, 1.0f, alpha);
		glTranslatef(180.0f, 220.0f, 60.0f);
		glBindTexture(GL_TEXTURE_2D, *txrs[txrs_current]);
		glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 1.0f);	glVertex3f(-128.0f,-128.0f, 0.0f);
			glTexCoord2f(0.0f, 0.0f);	glVertex3f(-128.0f, 128.0f, 0.0f);
			glTexCoord2f(1.0f, 0.0f);	glVertex3f( 128.0f, 128.0f, 0.0f);
			glTexCoord2f(1.0f, 1.0f);	glVertex3f( 128.0f,-128.0f, 0.0f);
		glEnd();
	}

	glKosFinishFrame();

	frame++;
}

// Call to do one iteration of the main loop
static bool demo_mainloop() {
	// Check to see if we have any user input
	if (check_control())
		return true;

	// Draw a frame of video update
	do_frame();

	return false;
}

// Do the "banner intro"
static bool banner_intro() {
	// Set the banner to ridiculously large and scroll it across
	float hw, hh;
	fnt->get_size(&hw, &hh, 400.0f, banner_text);
	banner_ps = 400.0f;
	banner_y = 240.0f;
	banner_alpha = 1.0f;
	for (banner_x = 640.0f + hw/2.0f; banner_x>=-hw/2.0f; banner_x-=32.0f) {
		if (demo_mainloop())
			return true;
	}

	banner_x = 180.0f; banner_y = 60.0f; banner_ps = 36.0f;
	for (banner_alpha = 0.0f; banner_alpha<1.0f; banner_alpha += 0.05f)
		if (demo_mainloop())
			return true;

	return false;
}

/////////////////////////////////////////////////////////////////////////////
// Lua functions

// Post some text up on the screen
bool demo_text(const char **msgs, int cnt) {
	if (text_cnt != 0) {
		// Fade it out
		text_fading = true;
		for (text_alpha=1.0f; text_alpha>=0.0f; text_alpha-=0.1f)
			if (demo_mainloop())
				return true;
		text_fading = false;
	
		for (int i=0; i<text_cnt; i++)
			free(text[i]);
	}
	text_cnt = cnt;
	
	if (text_cnt != 0) {
		// Copy the text from the Lua stack and figure out
		// the bounding box
		text_w = 0;
		for (int i=0; i<text_cnt; i++) {
			text[i] = strdup(msgs[i]);

			float w, h;
			fnt->get_size(&w, &h, 16.0f, text[i]);
			if (w > text_w)
				text_w = w;
		}

		text_h = text_cnt * 16.0f + 16.0f;
		text_w = text_w + 16.0f;
	}

	text_alpha = 0.0f;

	return false;
}

// Frame sleep
bool demo_sleep(int framecnt) {
	for (int i=0; i<framecnt; i++)
		if (demo_mainloop())
			return true;
	return false;
}

// Set the number of textures we'll be loading (in init)
void demo_txr_cnt(int cnt) {
	if (txrs_cnt != -1) {
		for (int i=0; i<txrs_cnt; i++)
			if (txrs[i])
				txrs[i]->unref();
		delete[] txrs;
	}

	txrs_cnt = cnt;
	txrs = new Texture * [cnt];
	for (int i=0; i<txrs_cnt; i++)
		txrs[i] = NULL;
}

// Load a texture (in init)
void demo_txr_load(int slot, const char * fn) {
	assert(slot < txrs_cnt);
	txrs[slot] = new Texture(fn, true);
	txrs[slot]->ref();
}

// Fade in / Fade out the Player object
bool demo_pv_fadein() {
	pv_visible = true;
	for (pv_alpha = 0.0f; pv_alpha <= 1.0f; pv_alpha+=0.1f) {
		if (demo_mainloop())
			return true;
	}
	return false;
}

bool demo_pv_fadeout() {
	for (pv_alpha = 1.0f; pv_alpha >= 0.0f; pv_alpha-=0.1f) {
		if (demo_mainloop())
			return true;
	}
	pv_visible = false;
	return false;
}

// Start the PV playing
bool demo_pv_start() {
	uint32 gt;
	
	// Save the old game time
	gt = gps->game_time;

	// Start a new play session
	gps->mode = GameplayState::ModeDemoPlay;
	gps->record_start();

	// Offset by the old time so there's no jump
	gps->start_s -= gt / 1000;
	gps->start_ms -= gt % 1000;
	if (gps->start_ms < 0) {
		gps->start_ms += 1000;
		gps->start_s--;
	}

	return false;
}

// Pause the PV
bool demo_pv_pause() {
	gps->mode = GameplayState::ModeDemoPause;
	return false;
}

// Set whether PV arrows will be hit
bool demo_pv_sethit(bool set) {
	gps->hitall = set;
	return false;
}

// Switch textures; if fade is true, then a fade effect
// will be used. Otherwise the switch will be immidiate.
bool demo_txr_switch(int idx, bool fade) {
	if (fade) {
		if (txrs_current != -1) {
			if (wtold) delete wtold;
			wtold = new WaveTexture(txrs[txrs_current], GL_LIST_TRANS_POLY);
			wtold->m_mag = 0.0f;
			wtold->m_alpha = 1.0f;
			wtold->set_translate(180.0f, 220.0f, 129.0f);
		}
		if (idx != -1) {
			if (wtnew) delete wtnew;
			wtnew = new WaveTexture(txrs[idx], GL_LIST_TRANS_POLY);
			wtnew->m_mag = 10.0f;
			wtnew->m_alpha = 0.0f;
			wtnew->set_translate(180.0f, 220.0f, 130.0f);
		}
	}
	txrs_current = idx;

	return false;
}


/////////////////////////////////////////////////////////////////////////////
// Init / Shutdown

// Create a set of step data for the demo
static StepData * create_steps() {
	// We need step data here with:
	// total length of 60 beats, gap 0
	// left arrow at beat 16
	// left/right arrow at beat 24
	// bomb arrow at beat 32
	// right arrow at beat 40
	StepData * sd = new StepData();
	char * sdchars = new char[120];
	for (int i=0; i<120; i++) {
		if (i == 119)
			sdchars[i] = 0;
		else
			sdchars[i] = '0';
	}
	sdchars[16*2] = '4';	// left
	sdchars[24*2] = 'B';	// left/right
	sdchars[32*2] = '8';	// up
	sdchars[40*2] = '6';	// right
	sd->step_data[StepData::Easy] = sdchars;

	// Setup a few other parameters
	sd->bpm = 180;
	sd->gap = 0;

	// Have it parse the data as normal
	sd->parse_data(StepData::Easy);

	// Add the step bomb at 32
	sd->steps[32*4].flags |= STEPFLAG_ITEM_BOMB;

	// All done
	return sd;
}

void do_demo() {
	//malloc_stats();

	pak_mount("demo", "gfx/demo");
	pak_mount("gameplay", "gfx/gameplay");
	pak_mount("fonts", "gfx/fonts");

	// Scrolling backgroun
	scrolly = new ScrollyBG("gfx/misc/scrolly_logo.png");
	scrolly->set_translate(0,0,10);

	// Font helper
	fnt = new FontHelper("gfx/fonts/axaxax.txf");

	// Setup our PV object
	GameplaySkin * gpskin = new GameplaySkin(false);
	StepData * steps = create_steps();
	gps = new GameplayState();
	gps->gap = steps->gap;
	gps->timeunit = steps->timeunit;
	gps->gap_used = true;
	gps->mode = GameplayState::ModeDemoPause;
	gps->game_time = 0;
	pv = new Player(NULL, gpskin, gps, 0, 640.0f - (256.0f + 32.0f), 32.0f, 60.0f, steps);
	pv_visible = false;

	// Setup blank WaveTexture objects for now
	wtnew = wtold = NULL;

	// Setup the banner
	banner_x = 1024.0f; banner_y = 600.0f; banner_ps = 14.0f;
	banner_alpha = 0.0f;

	// Let the Lua script set itself up
	foflua_call("demo_setup");

	pak_unmount("gfx/fonts");
	pak_unmount("gfx/gameplay");
	pak_unmount("gfx/demo");

	loading_finish();

	gfx_to_2d();
	glEnable(GL_TEXTURE_2D);

	// Start the music
	ogg_start("music/demo.ogg", false, false, false);

	frame = 0;

	// Fade in
	for (alpha=0.0f; alpha<=1.0f; alpha+=0.01f)
		demo_mainloop();
	alpha = 1.0f;

	// Do banner intro
	if (!banner_intro()) {
		// Call the Lua script for the middle part
		foflua_call("demo");

		// Go until there's no texture transition happening
		while (wtold || wtnew)
			demo_mainloop();
	}

	// Fade out
	for (alpha=1.0f; alpha>=0.0f; alpha-=0.05f) {
		do_frame();
		sndoggvorbis_volume((int)(alpha * 255));
	}
	alpha = 0.0f;

	// A few more to clear out the buffers
	do_frame();
	do_frame();
	do_frame();

	ogg_stop(false);

	// Clean up any remaining text messages
	for (int i=0; i<text_cnt; i++)
		free(text[i]);
	text_cnt = 0;

	// And any textures
	for (int i=0; i<txrs_cnt; i++)
		if (txrs[i])
			txrs[i]->unref();
	delete[] txrs;
	txrs = NULL;
	txrs_cnt = 0;

	delete fnt;
	delete scrolly;

	delete pv;
	// delete steps;	// pv does this
	delete gpskin;
	delete gps;

	loading_start();
}


