/*
   Feet of Fury

   editor/editor.cc

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#include "global.h"

/*

 PUBLIC SOURCE NOTE:

 This file is no longer used. It's one of the apocryphal pieces of FoF
 that never made it into the final cut due to time.

*/

/*

  This is some pretty hackish and nasty code in a lot of ways, but hey,
  it was quick to write and it works.

 */

static fs_builtin_ent bitab[] = {
	{ "/song.ogg", NULL, 0 }
};
static char last_ogg_name[256] = {0};

// Constructor / Destructor
Editor::Editor() {
	m_frame = 0;
	m_state = StateEditing;

	// Make sure the bitab is setup for our ogg buffer
	bitab[0].data = NULL;
	bitab[0].size = 0;
	last_ogg_name[0] = 0;
	fs_builtin_init();
	fs_builtin_set_table(bitab, 1);

	// Setup some blank steps
	strcpy(m_step_file, "editor/template.dwi");
	m_diff = StepData::Easy;
	m_steps = new StepData(m_step_file);
	m_steps->parseData(m_diff);

	m_gpstate.gap = m_steps->gap;
	m_gpstate.timeunit = m_steps->timeunit;
	m_gpstate.gap_used = true;
	m_gpstate.mode = GameplayState::ModeEdit;
	m_gpstate.syncTime();
	
	// Player visuals
	pakMount("gameplay", "gfx/gameplay");
	pakMount("fonts", "gfx/fonts");
	m_gpskin = new GameplaySkin(false);
	m_pvis = new Player(NULL, m_gpskin, &m_gpstate, 0, 32.0f, 64.0f, 10.0f, m_steps, NULL);
	pakUnmount("gfx/gameplay");
	pakUnmount("gfx/fonts");

	m_popup = NULL;

	m_limit = Limit16th;

	loading_finish();

	// Setup scene
	glClearColor(0.04f, 0.14f, 0.42f, 1.0f);
	gfx_to_2d();
}

Editor::~Editor() {
	loading_start();

	delete m_pvis;
	delete m_gpskin;
	// delete m_steps;	// pvis does this
	
	// Unload our sound effects
	snd_sfx_unload_all();

	// Free any bitab stuff
	fs_builtin_shutdown();
	if (bitab[0].data != NULL) {
		free(bitab[0].data);
		bitab[0].size = 0;
		bitab[0].data = NULL;
	}
	last_ogg_name[0] = 0;
}

// Called to actually do the editor
void Editor::do_editor() {
	while (m_state != StateDieing) {
		// Call all the per-frames
		control_per_frame();
		visual_per_frame();
	}

	// Give it a few more frames to clean the screen
	visual_per_frame();
	visual_per_frame();
	visual_per_frame();

	// Stop the music (if any is playing)
	sndoggvorbis_stop();
}

void Editor::draw_game_time() {
	char str[32];
	uint32 gt, m, s, ms;

	gt = m_gpstate.game_time;
	m = gt / (1000 * 60);
	s = (gt - (m*1000*60))/ 1000;
	ms = (gt - (m*1000*60)) % 1000;

	text_draw(12.0f, false, "Time", 0.0f, 1.0f, 1.0f);
	sprintf(str, "  Real %02ld:%02ld:%03ld", m, s, ms);
	text_draw(12.0f, false, str);
	sprintf(str, "  Beat %04d/%d", (int)(gt / (m_gpstate.timeunit*4)),
		(int)(gt / m_gpstate.timeunit) % 4);
	text_draw(12.0f, false, str);
	
	glKosPolyHdrDirty();
}

void Editor::control_popup() {
	m_popup->control_per_frame();
	if (m_popup->m_state == EditorPopup::StateDieing) {
		bool ab = m_popup->m_abort;
		delete m_popup;
		m_popup = NULL;
		m_state = StateEditing;
		if (ab) return;

		switch(m_popup_func) {
		case PopupLoad:
			delete m_steps;
			m_steps = new StepData(m_step_file);
			m_steps->parse_data(m_diff);
			m_gpstate.gap = m_steps->gap;
			m_gpstate.timeunit = m_steps->timeunit;
			m_gpstate.sync_time();
			m_pvis->set_step_data(m_steps);
			break;
		case PopupSave:
			m_steps->compile_data(m_diff);
			m_steps->save_to_file(m_step_file);
			break;
		case PopupConfClear:
			if (!strcmp(m_tmp_text, "yes")) {
				for (int i=0; i<m_steps->step_cnt; i++)
					m_steps->steps[i].pattern = '0';
			}
		case PopupGap:
			m_gpstate.gap = m_steps->gap = atoi(m_tmp_text);
			break;
		case PopupBPM:
			m_steps->bpm = atoi(m_tmp_text);
			m_steps->calc_bpm();
			m_gpstate.timeunit = m_steps->timeunit;
			m_gpstate.sync_time();
			break;
		case PopupMisc:
			break;
		}
	}
}

void Editor::control_playback() {
	int		i;

	m_gpstate.sync_time();

	// If the song ended, then stop playback and move to the top
	if (!sndoggvorbis_isplaying()) {
		m_gpstate.game_time = 0;
		switch_edit();
	}

	// Otherwise, process keys
	i = kbd_get_key();
	switch (i) {
	case 27:
		switch_edit();
		break;
	case (KBD_KEY_UP << 8):
		m_gpstate.gap -= 10;
		m_steps->gap -= 10;
		if (m_gpstate.gap_used) {
			if (m_gpstate.start_ms < 10) {
				m_gpstate.start_ms = 990 + m_gpstate.start_ms;
				m_gpstate.start_s--;
			} else {
				m_gpstate.start_ms -= 10;
			}
			m_gpstate.sync_time();
		}
		break;
	case (KBD_KEY_DOWN << 8):
		m_gpstate.gap += 10;
		m_steps->gap += 10;
		if (m_gpstate.gap_used) {
			m_gpstate.start_ms += 10;
			if (m_gpstate.start_ms >= 1000) {
				m_gpstate.start_ms -= 1000;
				m_gpstate.start_s++;
			}
			m_gpstate.sync_time();
		}
		break;
	case '1':
		m_limit = Limit4th;
		break;
	case '2':
		m_limit = Limit8th;
		break;
	case '3':
		m_limit = Limit16th;
		break;
	}
		
	// Check the controller; note that we only check the first
	// controller here, because otherwise debouncing becomes quite
	// difficult...
	maple_device_t	*addr;
	cont_state_t	*cst;
	int gidx = (int)(m_gpstate.game_time / m_gpstate.timeunit);

	// If we've got a limit, then round to the nearest step
	if (m_limit == Limit4th) {
		gidx = gidx & ~3;
	}
	if (m_limit == Limit8th) {
		gidx = gidx & ~1;
	}

	addr = maple_enum_type(0, MAPLE_FUNC_CONTROLLER);
	cst = (cont_state_t*)maple_dev_status(addr);
	// If there's something already there, ignore the change
	if (cst->buttons != m_last_btns) {
		if (cst->buttons & (CONT_A | CONT_B | CONT_X | CONT_Y
				| CONT_DPAD_LEFT | CONT_DPAD_RIGHT | CONT_DPAD_UP | CONT_DPAD_DOWN)) {
			int pattern = cst->buttons & (CONT_DPAD_LEFT | CONT_DPAD_RIGHT | CONT_DPAD_UP | CONT_DPAD_DOWN);
			if (cst->buttons & CONT_A)
				pattern |= CONT_DPAD_DOWN;
			if (cst->buttons & CONT_B)
				pattern |= CONT_DPAD_RIGHT;
			if (cst->buttons & CONT_X)
				pattern |= CONT_DPAD_LEFT;
			if (cst->buttons & CONT_Y)
				pattern |= CONT_DPAD_UP;
			m_steps->steps[gidx].pattern = Step::to_pattern(
				pattern | Step::to_cont(m_steps->steps[gidx].pattern));
		}
		m_last_btns = cst->buttons;
	}
}

void Editor::control_ed() {
	maple_device_t	*addr;
	cont_state_t	*cst;
	mouse_state_t	*mst;
	int		i;
	float		newgt, tu;

	newgt = m_gpstate.game_time;
	tu = m_gpstate.timeunit;

	// Any keys available?
	while ( (i=kbd_get_key()) != -1 ) {
		switch(i) {
		case (KBD_KEY_UP << 8):
			newgt = newgt - tu;
			break;
		case (KBD_KEY_DOWN << 8):
			newgt = newgt + tu;
			break;
		case (KBD_KEY_PGUP << 8):
			newgt = newgt - tu*16;
			break;
		case (KBD_KEY_PGDOWN << 8):
			newgt = newgt + tu*16;
			break;
		case (KBD_KEY_F1 << 8):
			if (m_diff != StepData::Easy)
				switch_data(StepData::Easy);
			break;
		case (KBD_KEY_F2 << 8):
			if (m_diff != StepData::Medium)
				switch_data(StepData::Medium);
			break;
		case (KBD_KEY_F3 << 8):
			if (m_diff != StepData::Hard)
				switch_data(StepData::Hard);
			break;
		case (KBD_KEY_F12 << 8):
			vid_screen_shot("/pc/tmp/ss.ppm");
			break;
		case 27:
			m_state = StateDieing;
			break;
		case 'l':
		case 'L':
			m_popup_func = PopupLoad;
			m_popup = new EditorPopup("Load DWI", m_step_file, m_gpskin);
			m_state = StatePopup;
			break;
		case 's':
		case 'S':
			m_popup_func = PopupSave;
			m_popup = new EditorPopup("Save DWI", m_step_file, m_gpskin);
			m_state = StatePopup;
			break;
		case 'n':
		case 'N':
			m_popup_func = PopupMisc;
			m_popup = new EditorPopup("Song Name", m_steps->song_name, m_gpskin);
			m_state = StatePopup;
			break;
		case 'a':
		case 'A':
			m_popup_func = PopupMisc;
			m_popup = new EditorPopup("Song Artist", m_steps->song_artist, m_gpskin);
			m_state = StatePopup;
			break;
		case 'o':
		case 'O':
			m_popup_func = PopupMisc;
			m_popup = new EditorPopup("Song's OGG File", m_steps->song_ogg, m_gpskin);
			m_state = StatePopup;
			break;
		case 'g':
		case 'G':
			m_popup_func = PopupGap;
			sprintf(m_tmp_text, "%d", m_steps->gap);
			m_popup = new EditorPopup("Gap Value", m_tmp_text, m_gpskin);
			m_state = StatePopup;
			break;
		case 'b':
		case 'B':
			m_popup_func = PopupBPM;
			sprintf(m_tmp_text, "%d", (int)m_steps->bpm);
			m_popup = new EditorPopup("BPM", m_tmp_text, m_gpskin);
			m_state = StatePopup;
			break;
		case 'p':
		case 'P':
			switch_playback();
			break;
		case 'c':
		case 'C':
			m_popup_func = PopupConfClear;
			strcpy(m_tmp_text, "no");
			m_popup = new EditorPopup("Enter 'yes' if you want to clear", m_tmp_text, m_gpskin);
			m_state = StatePopup;
			break;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
			ins_arrows(i - '0');
			break;
		case '!':
		case '@':
		case '#':
		case '$':
		case '%':
		case '^':
		case '&':
		case '*':
			rm_arrows(i);
			break;
		}
	}

	// Check the controller
	addr = maple_enum_type(0, MAPLE_FUNC_CONTROLLER);
	{
		cst = (cont_state_t*)maple_dev_status(addr);

		if (cst->buttons & CONT_START)
			m_state = StateDieing;
			
		int gidx = (int)(m_gpstate.game_time / m_gpstate.timeunit);

		if (cst->ltrig > 0)
			m_steps->steps[gidx].pattern = '0';

		if (cst->buttons & (CONT_A | CONT_B | CONT_X | CONT_Y)) {
			int btns = 0;
			if (cst->buttons & CONT_A)
				btns |= CONT_DPAD_DOWN;
			if (cst->buttons & CONT_B)
				btns |= CONT_DPAD_RIGHT;
			if (cst->buttons & CONT_X)
				btns |= CONT_DPAD_LEFT;
			if (cst->buttons & CONT_Y)
				btns |= CONT_DPAD_UP;
			m_steps->steps[gidx].pattern = Step::to_pattern(
				Step::to_cont(m_steps->steps[gidx].pattern) | btns);
		}

		if (cst->buttons & CONT_DPAD_UP) {
			if (m_last_btns != cst->buttons)
				newgt = newgt - tu;
		}
		if (cst->buttons & CONT_DPAD_DOWN) {
			if (m_last_btns != cst->buttons)
				newgt = newgt + tu;
		}

		if (cst->joyy < 0) {
			newgt = newgt - tu;
		}

		if (cst->joyy > 0) {
			newgt = newgt + tu;
		}

		m_last_btns = cst->buttons;
	}

	// Check the mouse
	i = 0;
	while ( (addr = maple_enum_type(i++, MAPLE_FUNC_MOUSE)) ) {
		mst = (mouse_state_t*)maple_dev_status(addr);
		if (mst->dz != 0) {
			newgt += mst->dz * 1000;
		}
	}

	// Sync up all the time things
	if (newgt < 0)
		newgt = 0;
	if (newgt > ((m_steps->step_cnt - 1) * m_gpstate.timeunit))
		newgt = (m_steps->step_cnt - 1) * m_gpstate.timeunit;
	if ((newgt - (int)newgt) > 0.5f)
		m_gpstate.game_time = (uint32)newgt+1;
	else
		m_gpstate.game_time = (uint32)newgt;
	m_gpstate.sync_time();
}

void Editor::control_per_frame() {
	switch (m_state) {
	case StateEditing:
		control_ed();
		break;
	case StatePopup:
		control_popup();
		break;
	case StatePlayback:
		control_playback();
		break;
	default:
		return;
	}
}

void Editor::visual_per_frame() {
	glKosBeginFrame();

	glKosFinishList();

	m_ty_pos = 20.0f;

	// Draw a banner of what's selected, if we're not moving
	text_draw(12.0f, true, "FoF Step Editor", 0.0f, 1.0f, 1.0f);
	text_skip(8.0f);

	char tmp[256];
	text_draw(12.0f, false, m_step_file);
	text_draw(12.0f, false, m_steps->song_name);
	text_draw(12.0f, false, m_steps->song_artist);
	text_draw(12.0f, false, m_steps->song_ogg);
	sprintf(tmp, "BPM: %d", (int)(m_steps->bpm));
	text_draw(12.0f, false, tmp);
	sprintf(tmp, "Gap: %d", m_steps->gap);
	text_draw(12.0f, false, tmp);
	if (m_state == StatePlayback) {
		if (m_gpstate.gap_used)
			text_draw(12.0f, false, "  (used)");
		else
			text_draw(12.0f, false, "  (not used)");
		switch (m_limit) {
		case Limit4th:
			text_draw(12.0f, false, "Limit to 1/4ths");
			break;
		case Limit8th:
			text_draw(12.0f, false, "Limit to 1/8ths");
			break;
		case Limit16th:
			text_draw(12.0f, false, "Limit to 1/16ths");
			break;
		}
	}
	switch (m_diff) {
	case StepData::Easy:
		text_draw(12.0f, false, "Basic/Easy");
		break;
	case StepData::Medium:
		text_draw(12.0f, false, "Trick/Medium");
		break;
	case StepData::Hard:
		text_draw(12.0f, false, "Maniac/Hard");
		break;
	}
	text_skip(8.0f);

	m_pvis->visual_per_frame(GL_LIST_TRANS_POLY);
	draw_game_time();
	// m_mouse.draw(GL_LIST_TRANS_POLY);

	text_skip(8.0f);
	text_usage();

	if (m_popup)
		m_popup->visual_per_frame();

	glKosFinishFrame();

	m_frame++;
}

// Draw a line of text on the status side
void Editor::text_draw(float size, bool centered, const char *s, float r, float g, float b) {
	float width, height;

	m_gpskin->fnt->begin();
	m_gpskin->fnt->get_size(&width, &height, size, s);
	m_gpskin->fnt->set_color(r, g, b);
	if (centered)
		m_gpskin->fnt->draw((320.0f + 160.0f) - width/2, m_ty_pos, 2.0f, size, s, 1.0f);
	else
		m_gpskin->fnt->draw(320.0f, m_ty_pos, 2.0f, size, s, 1.0f);
	m_gpskin->fnt->end();

	m_ty_pos += size+1;
}

// Skip some space
void Editor::text_skip(float amt) {
	m_ty_pos += amt;
}

// Draw usage info
void Editor::text_usage() {
	text_draw(12.0f, false, "Usage", 0.0f, 1.0f, 1.0f);
	text_skip(8.0f);
	if (m_state == StateEditing) {
		text_draw(12.0f, false, " Keyboard", 0.0f, 0.5f, 0.5f);
		text_draw(12.0f, false, "   Arrows: move 1/16th");
		text_draw(12.0f, false, "   PgUp/PgDn: move measure");
		text_draw(12.0f, false, "   ESC: quit");
		text_draw(12.0f, false, "   L: load DWI");
		text_draw(12.0f, false, "   S: save DWI");
		text_draw(12.0f, false, "   N/A/O/G/B: song attrs");
		text_draw(12.0f, false, "   P: test play");
		text_draw(12.0f, false, "   1-8: insert N blank 16ths");
		text_draw(12.0f, false, "   !-*: remove N 16ths");
		text_draw(12.0f, false, "   C: clear all steps");
		text_draw(12.0f, false, "   F1: basic/easy difficulty");
		text_draw(12.0f, false, "   F2: trick/medium difficulty");
		text_draw(12.0f, false, "   F3: maniac/hard difficulty");
		text_draw(12.0f, false, " Controller", 0.0f, 0.5f, 0.5f);
		text_draw(12.0f, false, "   D Pad: move 1/16th");
		text_draw(12.0f, false, "   Joystick: move faster");
		text_draw(12.0f, false, "   A/B/X/Y: set pattern");
		text_draw(12.0f, false, "   L Trigger: clear pattern");
		text_draw(12.0f, false, "   Start: quit");
	} else if (m_state == StatePlayback) {
		text_draw(12.0f, false, " Keyboard", 0.0f, 0.5f, 0.5f);
		text_draw(12.0f, false, "   ESC: stop playback");
		text_draw(12.0f, false, "   Up: sub 10ms from gap");
		text_draw(12.0f, false, "   Down: add 10ms to gap");
		text_draw(12.0f, false, "   1: limit to 1/4 notes");
		text_draw(12.0f, false, "   2: limit to 1/8 notes");
		text_draw(12.0f, false, "   3: limit to 1/16 notes");
		text_draw(12.0f, false, " Controller", 0.0f, 0.0f, 0.5f);
		text_draw(12.0f, false, "   D Pad: set pattern");
		text_draw(12.0f, false, "   A/B/X/Y: set pattern");
	}
}

// Switch to playback test mode
void Editor::switch_playback() {
	if (m_state == StatePlayback)
		return;

	if (strcmp(m_steps->song_ogg, last_ogg_name)) {
		if (bitab[0].data != NULL) {
			free(bitab[0].data);
			bitab[0].data = NULL;
			bitab[0].size = 0;
			last_ogg_name[0] = 0;
		}

		file_t f = fs_open(m_steps->song_ogg, O_RDONLY);
		if (!f) {
			printf("Can't read source ogg file '%s'\n", m_steps->song_ogg);
			return;
		}

		bitab[0].size = fs_total(f);
		bitab[0].data = (uint8*)malloc(bitab[0].size);
		fs_read(f, bitab[0].data, bitab[0].size);
		
		fs_close(f);

		strcpy(last_ogg_name, m_steps->song_ogg);
	}

	sndoggvorbis_queue_enable();
	if (sndoggvorbis_start("/bi/song.ogg", 0) < 0)
		return;
	sndoggvorbis_queue_wait();

	m_state = StatePlayback;
	m_last_btns = 0;
	m_gpstate.mode = GameplayState::ModeEditTest;
	m_gpstate.game_time = 0;
	m_gpstate.gap_used = false;
	m_gpstate.record_start();
	sndoggvorbis_queue_go();
}

// Switch back to edit mode from playback test mode
void Editor::switch_edit() {
	if (m_state == StateEditing)
		return;
		
	m_state = StateEditing;
	m_gpstate.mode = GameplayState::ModeEdit;
	m_gpstate.gap_used = true;

	// Re-normalize the time
	int gt = m_gpstate.game_time;
	int itu = (int)m_gpstate.timeunit;
	if (gt % itu) {
		gt = (gt / itu) * itu;
		m_gpstate.game_time = gt;
		m_gpstate.sync_time();
	}

	sndoggvorbis_stop();
}

// Switch which data set we're looking at
void Editor::switch_data(StepData::Difficulty d) {
	// Save back the old data
	m_steps->compile_data(m_diff);

	// Load up new data
	m_diff = d;
	m_steps->parse_data(m_diff);
}

// Remove N arrows at the current position
void Editor::rm_arrows(int cnt) {
	int cp = (int)(m_gpstate.game_time / m_gpstate.timeunit);

	// We need to translate cnt also, from a char...
	char chars[] = "!@#$%^&*";
	int i;
	for (i=0; i<8; i++)
		if (cnt == chars[i]) {
			cnt = i + 1;
			break;
		}
	assert(i < 8);
	
	m_steps->rm_arrows(cp, cnt);
}

// Insert N blank arrows at the current position
void Editor::ins_arrows(int cnt) {
	int cp = (int)(m_gpstate.game_time / m_gpstate.timeunit);
	m_steps->ins_arrows(cp, cnt);
}




// Call this from main to do the editor
void do_editor() {
	Editor editor;
	editor.do_editor();
}
