/*
   Feet of Fury

   gameplay/gpstate.cc

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

/*

 PUBLIC SOURCE NOTE:

 All typing and practice mode related pieces have been removed from the
 code so there's still _some_ motivation to buy the commercial one :)

*/
   
#include "global.h"
#include "gpstate.h"
#include "options.h"
#include "stepdata.h"

// Constructor / Destructor
GameplayState::GameplayState() {
	start_s = start_ms = 0;
	game_time = game_beat = 0;
	gap_used = false;
	state = StateNormal;
	frame = 0;
	alpha = 1.0f;
	gap = 0;
	timeunit = 0.0f;
	hitall = false;
	mode = ModePlay;
	next_bpm_change = NULL;
	scene = new Scene();
	scene->setTint(Color(1,1,1,1));

	for (int i=0; i<2; i++) {
		pp[i].life = 0.8f;
		pp[i].furiosity = 0.0f;
		for (int j=0; j<AttackCount; j++) {
			pp[i].attacks[j].active = false;
			pp[i].attacks[j].end = 0xffffffff;
			pp[i].attacks[j].counter = 0.0f;
		}
		pp[i].qnpixels = 64.0f;
		pp[i].attack_avail = AttackNone;
		pp[i].ai = false;
		pp[i].ai_accuracy = game_options.ai_skill / 100.0f;
		pp[i].unlock_pts = 0;
	}
}

GameplayState::~GameplayState() {
}

// Record the game start time; adjust for a negative gap here if neccessary
void GameplayState::recordStart() {
	uint32 s, ms;
	int gap2;
	
	timer_ms_gettime(&s, &ms);

	// Both players should have the same step data...
	if (gap < 0) {
		gap2 = -gap;
		s -= gap2 / 1000;
		ms -= gap2 % 1000;
		gap_used = true;
	}

	start_s = s;
	start_ms = ms;
}

// How many milliseconds have passed since the start time?
uint32 GameplayState::getGameTime() {
	uint32 s, ms, d;

	timer_ms_gettime(&s, &ms);
	d = (s - start_s) * 1000;
	d += (ms - start_ms);
	return d;
}

// Update game_time and game_beat
void GameplayState::syncTime() {
	if (mode == ModePlay || mode == ModeEditTest || mode == ModeDemoPlay)
		game_time = getGameTime();

	if (!gap_used && (int)game_time >= gap) {
		gap_used = true;
		start_s += gap / 1000;
		start_ms += gap % 1000;
		if (mode == ModePlay || mode == ModeEditTest || mode == ModeDemoPlay)
			game_time = getGameTime();
	}

	old_game_beat = game_beat;
	game_beat = (uint32)(game_time / timeunit);
}

// Given the current game time and song stats, calculate how many
// pixels "into" the song we should be at this point.
float GameplayState::pixelOffset(int index) {
	// Figure out what exact game beat we're at now (with fractional parts)
	float gbfloat = game_time / timeunit;

	// Each game beat is a 16th note, so divide that by 4
	gbfloat = gbfloat / 4.0f;

	// Now multiply that by the size of a quarter note
	return gbfloat * pp[index].qnpixels;
}

bool GameplayState::animArrows() {
	return mode == ModePlay || mode == ModeDemoPlay || mode == ModeDemoPause;
}
bool GameplayState::flashBlaze() {
	return mode == ModeEdit || mode == ModeEditTest;
}
bool GameplayState::measureBars() {
	return mode == ModeEdit || mode == ModeEditTest;
}
bool GameplayState::pxlOffsets() {
	return mode == ModePlay || mode == ModeEditTest || mode == ModeDemoPlay;
}
bool GameplayState::fullPlayfield() {
	return mode == ModePlay || mode == ModeDemoPlay || mode == ModeDemoPause;
}
bool GameplayState::selector() {
	return mode == ModeEdit || mode == ModeEditTest;
}
bool GameplayState::meters() {
	return mode == ModePlay;
}
bool GameplayState::judge() {
	return mode == ModePlay;
}
bool GameplayState::drawBackground() {
	return mode == ModePlay;
}
bool GameplayState::allowAttacks() {
	return true;
}
bool GameplayState::doWinner() {
	return true;
}
bool GameplayState::showCharacters() {
	return mode == ModePlay;
}

