/*      
   Feet of Fury
        
   editor.h

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

/*

 PUBLIC SOURCE NOTE:

 This file is no longer used. It's one of the apocryphal pieces of FoF
 that never made it into the final cut due to time.

*/


#ifndef __EDITOR_H
#define __EDITOR_H

class Editor {
public:
	Editor();
	virtual ~Editor();

	// Called to actually do the editor
	void doEditor();

	// Game time display
	void drawGameTime();

	// Control input per-frame for different modes
	void controlPerFrame();
	void controlEd();
	void controlPopup();
	void controlPlayback();

	// Visuals per-frame
	void visualPerFrame();

	// Draw a line of text on the status side
	void textDraw(float size, bool centered, const char *s, float r=1.0f, float g=1.0f, float b=1.0f);

	// Skip some space
	void textSkip(float amt);

	// Draw usage info
	void textUsage();

	// Switch to playback test mode
	void switchPlayback();

	// Switch back to edit mode from playback test mode
	void switchEdit();

	// Switch which data set we're looking at
	void switchData(StepData::Difficulty d);

	// Remove N arrows at the current position
	void rmArrows(int cnt);

	// Insert N blank arrows at the current position
	void insArrows(int cnt);

private:
	// Font renderer
	RefPtr<Font>	m_fnt;

	// Frame ticker (to know when to change states, etc)
	int		m_frame;

	// Mouse pointer
	// Mouse		m_mouse;

	// Current steps we're working on
	char		m_step_file[256];
	char		m_ogg_file[256];
	StepData	* m_steps;

	// "Gameplay State" for manipulating the player visuals
	GameplayState	m_gpstate;

	// PlayerVisuals (for arrows)
	GameplaySkin	* m_gpskin;
	Player		* m_pvis;

	// If we have a text input popup, this points to it
	enum PopupFunc { PopupLoad, PopupSave, PopupConfClear, PopupGap, PopupBPM, PopupMisc };
	PopupFunc	m_popup_func;
	EditorPopup	* m_popup;
	char		m_tmp_text[256];

	// Our state
	enum State { StateEditing, StatePopup, StatePlayback, StateDieing };
	State		m_state;

	// Text cursor position
	float		m_ty_pos;

	// During record mode, we'll limit to what kind of note?
	enum Limit { Limit4th, Limit8th, Limit16th };
	Limit		m_limit;

	// Last button pattern (during playback) for debouncing
	uint32		m_last_btns;

	// What difficulty are we editing?
	StepData::Difficulty	m_diff;
};

// Call this from main to do the main menu
void doEditor();

#endif	/* __EDITOR_H */

