/*      
   Feet of Fury
        
   loading.h

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#ifndef __LOADING_H
#define __LOADING_H

class Bubble;
class Cube;

class Loading {
public:
	Loading();
	virtual ~Loading();

	// Do a frame of display
	void doFrame();

	// Our thread will run in here
	void threadproc();

	// Tell us to begin our fun display
	void start();

	// Tell us to stop our fun display
	void finish();

private:
	// Signal / wait on the semaphore
	void signal();
	void wait();

	// Methods to draw the various rings
	void drawOuterRing();
	void drawMidRing();
	void drawInnerRing();

	// Draw the "throbber" cube/sphere
	void drawThrobber();

	// Draw the "loading" ring
	void drawLoadingRing();

	volatile enum State {
		StateWaitBirth,
		StateReady,
		StateRunning,
		StateStopRunning,
		StateWaitDeath,
		StateDead
	} m_state;

	semaphore_t	* m_sem;
	
	RefPtr<Bubble>	m_bubble;
	RefPtr<Cube>	m_cube;
	RefPtr<Texture>	m_txr, m_txrLoading;

	int		m_frame;
	float		m_alpha;
};

// Call to initialize/shutdown the loading twiddler
void loadingInit();
void loadingShutdown();

// Call when you start loading
void loadingStart();

// Call when you're done loading
void loadingFinish();

#endif	/* __LOADING_H */

