/*      
   Feet of Fury
        
   step.h

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.

   $Id: step.h,v 1.12 2003/03/02 07:11:55 bard Exp $
*/

#ifndef __STEP_H
#define __STEP_H

#define STEPFLAG_ADDED		0x0001	// Step was added due to a bomb
#define STEPFLAG_ERASED		0x0002	// Step was erased
#define STEPFLAG_ITEM_MASK	0xff00	// Mask for item flags
#define STEPFLAG_ITEM_BOMB	0x0100	// Arrow bomb
#define STEPFLAG_ITEM_DRUNKEN	0x0200	// Drunken arrow
#define STEPFLAG_ITEM_VORTEX	0x0300	// Vortex
#define STEPFLAG_ITEM_TURBO	0x0400	// Turbo
#define STEPFLAG_ITEM_PHASING	0x0500	// Phasing arrows
#define STEPFLAG_ITEM_FIREBALL	0x0600	// Fireball
#define STEPFLAG_ITEM_HEALTH	0x1000	// +Health
#define STEPFLAG_ITEM_ERASER	0x1100	// Arrow eraser
#define STEPFLAG_ITEM_DISPEL	0x1200	// Dispel
#define STEPFLAG_ITEM_REVERSE	0x1300	// Reverse

// Small struct with arrow specs; there's one of these per
// potential arrow position in the step data.
struct Step {
	Step() {
		blank();
	}
	int	pattern;	// Arrow pattern
	int	ch;		// Character for typing mode
	int	d;		// Dead-on time (in ms)
	int	flags;		// From the defines
	float	fade;		// Fade percent 0..1 for erased arrows

	static uint32	toCont(int pat) {
		switch(pat) {
		case '0':
			return 0;
		case '8':
			return CONT_DPAD_UP;
		case '4':
			return CONT_DPAD_LEFT;
		case '6':
			return CONT_DPAD_RIGHT;
		case '2':
			return CONT_DPAD_DOWN;
		case '7':
			return CONT_DPAD_UP | CONT_DPAD_LEFT;
		case '9':
			return CONT_DPAD_UP | CONT_DPAD_RIGHT;
		case '1':
			return CONT_DPAD_DOWN | CONT_DPAD_LEFT;
		case '3':
			return CONT_DPAD_DOWN | CONT_DPAD_RIGHT;
		case 'A':
			return CONT_DPAD_UP | CONT_DPAD_DOWN;
		case 'B':
			return CONT_DPAD_LEFT | CONT_DPAD_RIGHT;
		case 'C':
			return CONT_DPAD_UP | CONT_DPAD_DOWN | CONT_DPAD_RIGHT;
		case 'D':
			return CONT_DPAD_UP | CONT_DPAD_DOWN | CONT_DPAD_LEFT;
		case 'E':
			return CONT_DPAD_DOWN | CONT_DPAD_LEFT | CONT_DPAD_RIGHT;
		case 'F':
			return CONT_DPAD_UP | CONT_DPAD_LEFT | CONT_DPAD_RIGHT;
		case 'G':
			return CONT_DPAD_UP | CONT_DPAD_DOWN | CONT_DPAD_LEFT | CONT_DPAD_RIGHT;
		default:
			printf("Invalid step data: %c / %d\n", pat, pat);
			// assert_msg( false, "Invalid step data" );
			return 0;
		}
	}

	static int toPattern(uint32 cont) {
		switch(cont) {
		case 0:
			return '0';
		case CONT_DPAD_UP:
			return '8';
		case CONT_DPAD_LEFT:
			return '4';
		case CONT_DPAD_RIGHT:
			return '6';
		case CONT_DPAD_DOWN:
			return '2';
		case CONT_DPAD_UP | CONT_DPAD_LEFT:
			return '7';
		case CONT_DPAD_UP | CONT_DPAD_RIGHT:
			return '9';
		case CONT_DPAD_DOWN | CONT_DPAD_LEFT:
			return '1';
		case CONT_DPAD_DOWN | CONT_DPAD_RIGHT:
			return '3';
		case CONT_DPAD_UP | CONT_DPAD_DOWN:
			return 'A';
		case CONT_DPAD_LEFT | CONT_DPAD_RIGHT:
			return 'B';
		case CONT_DPAD_UP | CONT_DPAD_DOWN | CONT_DPAD_RIGHT:
			return 'C';
		case CONT_DPAD_UP | CONT_DPAD_DOWN | CONT_DPAD_LEFT:
			return 'D';
		case CONT_DPAD_DOWN | CONT_DPAD_LEFT | CONT_DPAD_RIGHT:
			return 'E';
		case CONT_DPAD_UP | CONT_DPAD_LEFT | CONT_DPAD_RIGHT:
			return 'F';
		case CONT_DPAD_UP | CONT_DPAD_DOWN | CONT_DPAD_LEFT | CONT_DPAD_RIGHT:
			return 'G';
		default:
			printf("Invalid step data: %08lx\n", cont);
			// assert_msg( false, "Invalid step data" );
			return -1;
		}
	}

	void blank() {
		pattern = '0';
		ch = '\0';
		d = -1;
		flags = 0;
		fade = 1.0f;
		hit = NotYet;
	}

	enum Hit {
		NotYet = 0, Miss,
		Boo, Good, Great, Perfect,
		Added		// Overload this a bit
	}	hit;
};

#endif	/* __STEP_H */

