/*
   Feet of Fury

   lua.cc

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

/*

 PUBLIC SOURCE NOTE:

 This file is no longer used. It's one of the apocryphal pieces of FoF
 that never made it into the final cut due to time.

*/


#include "global.h"

/*

  Setup stuff for Lua usage. We'll keep everything Lua specific in this
  tree so that it can be easily removed later if we change our minds,
  and this also helps reduce namespace cluttering.

  This file contains all the basics and setup/shutdown. The other files
  in this tree provide specific support for various modules that use Lua
  in FoF.

*/

#include <lua/lua.h>

// Our global Lua state
static lua_State *ls = NULL;

// Custom fputs function to capture stdout of Lua
static void lua_print(const char *str) {
	printf("LUA: %s", str);
}

// Setup Lua
bool lua_init() {
	// Do the initial basic setup
	luaB_set_fputs(lua_print);
	ls = lua_open(0);	// Create status, no default stack size
	if (!ls) {
		printf("lua_init: Couldn't create state\n");
		return false;
	}

	// Hook in the Lua stdlibs
	lua_baselibopen(ls);
	lua_iolibopen(ls);
	lua_strlibopen(ls);
	lua_mathlibopen(ls);
	lua_dblibopen(ls);

	// Setup our FoF hooks
	foflua_setup_demo(ls);

	// Load up the FoF Lua file
	if (!lua_load("script/fof.lua"))
		return false;

	// Call a Lua test function
	/* printf("lua_init: top is %d\n", lua_gettop(ls));
	printf("lua_init: calling dodemo()\n");
	lua_getglobal(ls, "dodemo");
	lua_call(ls, 0, 1);
	printf("lua_init: result was of type '%s'\n",
		lua_typename(ls, lua_type(ls, lua_gettop(ls))));
	printf("lua_init: result was '%s'\n",
		lua_tostring(ls, lua_gettop(ls)));
	lua_pop(ls, 1);
	printf("lua_init: top is %d\n", lua_gettop(ls)); */

	// Ok, all done!
	return true;
}

// Shut down Lua
void lua_shutdown() {
	if (!ls)
		return;

	lua_close(ls);
}

// Load a Lua script from disk into our state (and execute it)
bool lua_load(const char *fn) {
	// Save our old stack position
	int top = lua_gettop(ls);

	// Execute the file load
	int res = lua_dofile(ls, fn);

	// Restore stack position
	lua_settop(ls, top);

	// Check for error
	if (res == LUA_ERRMEM) {
		printf("lua_load: memory allocation error while executing '%s'\n", fn);
		return false;
	} else if (res == LUA_ERRERR) {
		printf("lua_load: error in error message while executing '%s'\n", fn);
		return false;
	} else if (res == LUA_ERRFILE) {
		printf("lua_load: cannot execute file '%s'\n", fn);
		return false;
	} else if (res != 0) {
		printf("lua_load: error %d while executing '%s'\n", res, fn);
		return false;
	}

	// That pretty much exhausts the possibilities we care about.. return success
	return true;
}

