/*
   Feet of Fury

   lua_demo.cc

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#include "global.h"

/*

 PUBLIC SOURCE NOTE:

 This file is no longer used. It's one of the apocryphal pieces of FoF
 that never made it into the final cut due to time.

*/

/*

  Lua hooks for "demo" mode

*/

#include <lua/lua.h>

static lua_State *ls;

static int boolrt(lua_State * st, bool rv) {
	if (!rv)
		lua_pushnil(st);
	else
		lua_pushnumber(st, 1);
	return 1;
}

static int foflua_demo_sleep(lua_State *st) {
	// Check number of arguments
	assert( lua_gettop(st) == 1 );

	// Check the type
	assert( lua_isnumber(st, 1) );

	// Grab that number value
	int n = (int)lua_tonumber(st, 1);

	// Call the C++ func
	bool rv = demo_sleep(n);

	// Push the return value on (nil/1 for false/true)
	return boolrt(st, rv);
}

// Hook for the "display text" command
static int foflua_demo_text(lua_State * st) {
	// Check number of arguments
	assert( lua_gettop(st) == 1 );

	// Check the type
	assert( lua_istable(st, 1) );

	// Grab the table and figure out how many indeces it
	// has; we'll setup a temp string array and pass this
	// into the actual C++ demo code.
	int n = lua_getn(st, 1);
	const char * strs[n];

	// Now pull the individual strings
	for (int i=1; i<=n; i++) {
		lua_rawgeti(st, 1, i);
		assert( lua_isstring(st, lua_gettop(ls)) );

		strs[i-1] = lua_tostring(st, lua_gettop(ls));
	}

	// Call the C++ func
	bool rv = demo_text(strs, n);

	// Pop all that off the stack
	lua_settop(st, 1);

	// Push the return value on (nil/1 for false/true)
	return boolrt(st, rv);
}

// The rest of these are gonna be a lot less verbose...

static int foflua_demo_txr_cnt(lua_State * st) {
	assert( lua_gettop(st) == 1 );
	assert( lua_isnumber(st, 1) );
	demo_txr_cnt((int)lua_tonumber(st, 1));
	return 0;
}

static int foflua_demo_txr_load(lua_State * st) {
	assert( lua_gettop(st) == 2 );
	assert( lua_isnumber(st, 1) );
	assert( lua_isstring(st, 2) );
	demo_txr_load(
		(int)lua_tonumber(st, 1),
		lua_tostring(st, 2)
	);
	return 0;
}

static int foflua_demo_pv_fadein(lua_State * st) {
	assert( lua_gettop(st) == 0 );
	return boolrt(st, demo_pv_fadein());
}

static int foflua_demo_pv_fadeout(lua_State * st) {
	assert( lua_gettop(st) == 0 );
	return boolrt(st, demo_pv_fadeout());
}

static int foflua_demo_pv_start(lua_State * st) {
	assert( lua_gettop(st) == 0 );
	return boolrt(st, demo_pv_start());
}

static int foflua_demo_pv_pause(lua_State * st) {
	assert( lua_gettop(st) == 0 );
	return boolrt(st, demo_pv_pause());
}

static int foflua_demo_pv_sethit(lua_State * st) {
	assert( lua_gettop(st) == 1 );
	assert( lua_isnumber(st, 1) );
	return boolrt(st, demo_pv_sethit(lua_tonumber(st, 1) != 0));
}

static int foflua_demo_txr_switch(lua_State * st) {
	assert( lua_gettop(st) == 2 );
	assert( lua_isnumber(st, 1) );
	assert( lua_isnumber(st, 2) );

	return boolrt(st, demo_txr_switch(
		(int)lua_tonumber(st, 1),
		lua_tonumber(st, 2) != 0 ));
}

// Setup hooks for the demo screen
bool foflua_setup_demo(lua_State * state) {
	ls = state;
	lua_register(state, "demo_text", foflua_demo_text);
	lua_register(state, "demo_sleep", foflua_demo_sleep);
	lua_register(state, "demo_txr_cnt", foflua_demo_txr_cnt);
	lua_register(state, "demo_txr_load", foflua_demo_txr_load);
	lua_register(state, "demo_pv_fadein", foflua_demo_pv_fadein);
	lua_register(state, "demo_pv_fadeout", foflua_demo_pv_fadeout);
	lua_register(state, "demo_pv_start", foflua_demo_pv_start);
	lua_register(state, "demo_pv_pause", foflua_demo_pv_pause);
	lua_register(state, "demo_pv_sethit", foflua_demo_pv_sethit);
	lua_register(state, "demo_txr_switch", foflua_demo_txr_switch);
	return true;
}

// Called from the demo to run the Lua script
bool foflua_call(const char *fn) {
	lua_getglobal(ls, fn);
	return lua_call(ls, 0, 0) == 0;
}

