/*
   Feet of Fury

   cache.cc

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#include "global.h"
#include "cache.h"

#include <kos/fs_ramdisk.h>

// Cache utilities (uses the fs_ramdisk)

// This is like fs_copy, but optimized for fs_ramdisk to avoid
// memory fragmentation.
static int cacheCopy(const char * src, const char * dest) {
	void * buf;
	int size;

	size = fs_load(src, &buf);
	if (size < 0) {
		printf("cacheCopy: unable to load src '%s'\n", src);
		return -1;
	}

	if (fs_ramdisk_attach(dest+4, buf, size) < 0) {
		printf("cacheCopy: unable to attach from src '%s' to '%s'\n", src, dest+4);
		free(buf);
		return -1;
	}

	return 0;
}

int cacheLoad(const char * fn, const char * cachename) {
	char sfn[64];
	char cfn[64];

	if (fn[0] != '/')
		sprintf(sfn, "%s/%s", fs_getwd(), fn);
	else
		strcpy(sfn, fn);
	sprintf(cfn, "/ram/%s", cachename);

	printf("cache_load: cacheing file %s to %s\n", sfn, cfn);
	if (cacheCopy(sfn, cfn) < 0) {
		printf("cache_load: unable to cache file %s to %s\n", sfn, cfn);
		return -1;
	} else {
		return 0;
	}
}

int cacheUnload(const char * cachename) {
	char cfn[64];

	sprintf(cfn, "%s%s", CACHE_PREFIX, cachename);

	if (fs_unlink(cfn) < 0) {
		printf("cache_unload: unable to uncache %s\n", cfn);
		return -1;
	} else {
		printf("cache_unload: uncached %s\n", cfn);
		return -1;
	}
}

