/*
   Feet of Fury

   cdtray.cc

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#include "global.h"
#include "main_cdtray.h"

static semaphore_t * quit_sem;
static kthread_t * thd_hnd;
static volatile bool cd_allow_open;
static volatile int cd_status;
static void (*cb)(void *, int, int);
static void *cb_token;

static int vbl_hnd;

/* This is our quick checker that looks for an open tray at bad times,
   and updates the actual disc status. */
static void cdtVbl(uint32 code) {
	int rv, dt;
	rv = cdrom_get_status((int *)&cd_status, &dt);
	if (rv >= 0) {
		if (!cd_allow_open && cd_status == CD_STATUS_OPEN) {
			printf("cdtVbl: detected CD tray open without permission\n");
			printf("   (user signal to exit)\n");
			arch_exit();
		}
	}
}

static void cdtThreadproc(void *p) {
	int last_status = -1;

	printf("cdtThreadProc: started\n");
	do {
		/* int rv, dt;
		rv = cdrom_get_status((int *)&cd_status, &dt); */

		/* if (!cd_allow_open && cd_status == CD_STATUS_OPEN) {
			printf("cdtThreadProc: detected CD tray open without permission\n");
			printf("   (user signal to exit)\n");
			arch_exit();
		} */

		if (last_status != cd_status) {
			if (cb)
				cb(cb_token, last_status, cd_status);
			last_status = cd_status;
		}
	} while (sem_wait_timed(quit_sem, 250) == -1);
	printf("cdtThreadproc: finished\n");
}

void cdtrayInit() {
	quit_sem = sem_create(0);
	cd_status = -1;
	cd_allow_open = false;
	cb = NULL;
	cb_token = NULL;
	vbl_hnd = vblank_handler_add(cdtVbl);
	thd_hnd = thd_create(cdtThreadproc, NULL);
}

void cdtrayShutdown() {
	vblank_handler_remove(vbl_hnd);
	sem_signal(quit_sem);
	thd_wait(thd_hnd);
	sem_destroy(quit_sem);
}

void cdtrayAllowOpen(bool rv) {
	cd_allow_open = rv;
}

int cdtrayStatus() {
	return cd_status;
}

void cdtraySetCallback(void (*icb)(void *, int, int), void * token) {
	cb_token = token;
	cb = icb;
}
