/*
   Feet of Fury

   gfx.cc

   Copyright (C)2002,2003 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#include "global.h"

int hz = 60;

static pvr_init_params_t params = {
	/* Enable opaque and translucent polygons with size 32 */
	{ PVR_BINSIZE_32, PVR_BINSIZE_0, PVR_BINSIZE_32, PVR_BINSIZE_0, PVR_BINSIZE_0 },

	/* Vertex buffer size 256K */
	256*1024
};

bool gfxSwitchTo50() {
	pvr_wait_ready();
	pvr_shutdown();
	vid_shutdown();
	vid_init(DM_640x480_PAL_IL, PM_RGB565);
	if (pvr_init(&params) < 0)
		return false;
	hz = 50;
	return true;
}

bool gfxSwitchTo60() {
	pvr_wait_ready();
	pvr_shutdown();
	vid_shutdown();
	vid_init(DM_640x480_NTSC_IL, PM_RGB565);
	if (pvr_init(&params) < 0)
		return false;
	hz = 60;
	return true;
}

bool gfxIsVga() {
	int result = vid_check_cable();

	// RGB should be included here as it never needs a 50/60Hz switch...(??)
	return result == CT_VGA;
}

bool gfxInit() {
	/* Initialize PVR */
	if (pvr_init(&params) < 0)
		return false;

	// gfxSwitchTo50();

	/* Initialize the Parallax as well */
	plx_cxt_init();
	plx_mat3d_init();
	plx_mat3d_mode(PLX_MAT_PROJECTION);
	plx_mat3d_identity();
	plx_mat3d_perspective(45.0f, 640.0f / 480.0f, 0.1f, 100.0f);
	plx_mat3d_mode(PLX_MAT_MODELVIEW);

	return true;
}

void gfxShutdown() {
	pvr_stats_t	stats;

	pvr_get_stats(&stats);
	dbglog(DBG_DEBUG, "3D Stats: %ld vblanks, frame rate ~%f fps, max vertex used %d bytes\n",
		stats.vbl_count, (double)stats.frame_rate, stats.vtx_buffer_used_max);
}

