/*
   Feet of Fury

   main/misc.cc

   Copyright (C)2003 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#include "global.h"
#include "pak.h"
#include "stepdata.h"
#include "vmu.h"
#include "main_cdtray.h"
#include "loading.h"
#include "region.h"
#include "menu_5060hz.h"
#include "options.h"
#include "bsod.h"

#include <kosh/kosh.h>
#include <conio/conio.h>

// extern "C" uint8 romdisk[];
#if defined(RUN_FROM_PC) || defined(TEST_FROM_PC)
	KOS_INIT_FLAGS(INIT_DEFAULT | INIT_MALLOCSTATS);
#else
	KOS_INIT_FLAGS(INIT_DEFAULT | INIT_NO_DCLOAD | INIT_QUIET);
#endif
// KOS_INIT_ROMDISK(romdisk);

bool swapcd_enabled = false;
char swapcd_path[256];

extern "C" { void irq_dump_regs(int, int); }
void bail_handler(uint8 addr, uint32 btns) {
	irq_dump_regs(0,0);
	arch_exit();
}

#include <dc/sound/sound.h>
void memchecks() {
	printf("\nMemory Checks:\n\n");
	// pvr_mem_print_list();
	pvr_mem_stats();

	printf("Available sound ram: %d/%d bytes\n",
		(int)snd_mem_available(), 2048*1024);
}


StepData * loadSteps(const char *fn) {
	StepData * sd;

	char tmp[256];
	sprintf(tmp, "song_meta/%s/steps.dwi", fn);
	sd = new StepData(tmp, fn);

	return sd;
}

/////////////////////////////////////////////////////////////////////////////////
// Initial loading -- these things will get loaded at game boot
// time and will never be unloaded. A loading throbber is not displayed.
void loadInitial() {
	pakLoad("preload");

	// Actually mount the preload; and since we'll never unmount it, just
	// unload the compressed file.
	pakMount("preload");
	pakUnload("preload");
}

static volatile bool lft_thd_done = false;
static kthread_t * lft_thd = NULL;
static void loadFullThread(void *foo) {
	printf("***Beginning background load***\n");
	pakLoad("menu_char");
	pakLoad("menu_stage");
	pakLoad("gameplay");
	lft_thd_done = true;
	printf("***Finished background load***\n");
}

// These things get loaded after we get the basic setup going (loaded preloads,
// passed 50/60Hz selection, initted throbber, etc).
void loadFull() {
	// Load the titles pak with the throbber enabled
	loadingStart();
	pakLoad("titles");
	loadingFinish();

	// Finish loading everything else in a low prio thread
	lft_thd_done = false;
	lft_thd = thd_create(loadFullThread, NULL);
#if !defined(RUN_FROM_PC) && !defined(TEST_FROM_PC)
	// dc-load-ip no likey so you die!
	thd_set_prio(lft_thd, PRIO_DEFAULT+1);
#endif
}

// Ensure the thread completed before we go on
void loadFullComplete() {
	// This would be a race condition, but we can cancel the throbber
	// as soon as it's started and the user pretty much won't see anything.
	if (!lft_thd_done)
		loadingStart();
	thd_wait(lft_thd);
	loadingFinish();
	lft_thd = NULL;
}

/////////////////////////////////////////////////////////////////////////////////

int globalSetup() {
	// For CD release
#if !defined(RUN_FROM_PC) && !defined(TEST_FROM_PC)
	arch_set_exit_path(ARCH_EXIT_MENU);
	setupBsod();
#endif

	atexit(memchecks);

	// cont_btn_callback(0, CONT_START | CONT_A | CONT_B | CONT_X | CONT_Y, bail_handler);

#ifdef RUN_FROM_PC
	conio_init(CONIO_TTY_SERIAL, CONIO_INPUT_LINE);
	kosh_init();
#endif

	if (!pakInit())
		return -1;

	if (!gfxInit())
		return -1;

	/* Print a banner */
	printf("\n<-- Feet of Fury START ------------------------------------>\n\n");

	vmuInit();
	cdtrayInit();

	srand(time(NULL));

	// This is where we'll start from in the file system
#if defined(RUN_FROM_PC) || defined(TEST_FROM_PC)
	// For CD image testing
	//fs_chdir("/pc/fof/dcload_root");
	fs_chdir("/pc/fof/cd_root/fof");
#else
	// For CD release
	fs_chdir("/cd/fof");
#endif

	// Setup OggVorbis thread
	sndoggvorbis_init();

	// Setup Loading object
	loadInitial();

	// If we're in Europe and not running on a VGA box, ask the user if
	// they want 50 or 60 Hz mode.
#ifndef BUILD_5060HZ
	if (regionCheck() == RegionEurope && !gfxIsVga()) {
#endif
		doMenu5060Hz();
#ifndef BUILD_5060HZ
	}
#endif

	loadingInit();

	// Initialize Lua
	//lua_init();

	return 0;
}

int globalShutdown() {
	// Shut down Lua
	//lua_shutdown();

	// Shut down Loading object
	loadingShutdown();

	// Shut down
	sndoggvorbis_shutdown();

	vmuShutdown();
	cdtrayShutdown();

	// Print a banner
	gfxShutdown();

	pakShutdown();

	printf("\n<-- Feet of Fury FINISH ----------------------------------->\n\n");

#ifdef RUN_FROM_PC
	conio_shutdown();
	kosh_shutdown();
#endif

	return 0;
}


