/*
   Feet of Fury

   title.cc

   Copyright (C)2002 Cryptic Allusion, LLC

   See the file LICENSE for more info on usage.
*/

#include "global.h"
#include "res.h"
#include "title.h"
#include "ogg.h"

#include "scrollybg.h"

// Textures
static RefPtr<Texture> txr_logo;

// Scrolly bg
static RefPtr<ScrollyBG> scrolly;

// Font helper
static RefPtr<Font> fnt;

// Frame counter
static int frame;

// Alpha value
static float alpha;

// When did we start? (timeout tracking)
static uint32 start_s, start_ms;

static int checkCont() {
	maple_device_t	*addr;
	cont_state_t	*st;
	int		i = 0;

	while ( (addr = maple_enum_type(i++, MAPLE_FUNC_CONTROLLER)) ) {
		st = (cont_state_t*)maple_dev_status(addr);
		if ((st->buttons & (CONT_A | CONT_B | CONT_X | CONT_Y | CONT_START)) ==
			(CONT_A | CONT_B | CONT_X | CONT_Y | CONT_START))
		{
			return -1;
		}
		if (st->buttons & CONT_START)
			return 1;
	}

	MAPLE_FOREACH_BEGIN(MAPLE_FUNC_KEYBOARD, kbd_state_t, st)
		if (st->matrix[KBD_KEY_ESCAPE] || 
			st->matrix[KBD_KEY_ENTER])
		{
			return 1;
		}
		
		if (st->matrix[KBD_KEY_DEL] && (st->shift_keys & (KBD_MOD_LCTRL | KBD_MOD_RCTRL)) &&
				(st->shift_keys & (KBD_MOD_LALT | KBD_MOD_RALT)))
		{
			return -1;
		}
	MAPLE_FOREACH_END()

	uint32 s, ms;
	timer_ms_gettime(&s, &ms);
	if ((s - start_s) > 60)
		return 2;
	
	return 0;
}

static void drawFrame() {
	if (alpha < 0.0f)
		alpha = 0.0f;

	pvr_wait_ready();
	pvr_scene_begin();
	pvr_list_begin(PVR_LIST_OP_POLY);

	scrolly->setTint(Color(alpha,1,1,1));
	scrolly->draw(PLX_LIST_OP_POLY);
	scrolly->nextFrame();

	pvr_list_begin(PVR_LIST_TR_POLY);

	uint32 color = plx_pack_color(alpha, 1.0f, 1.0f, 1.0f);

	txr_logo->sendHdr(PLX_LIST_TR_POLY);
	mat_identity();
	mat_translate(320.0f, 200.f, 15.0f);
	mat_rotate_z( ((alpha * -90.0f) - 270.0f) * 2*M_PI / 360.0f );
	plx_spr_imp(
		2.0f * alpha * 256,
		2.0f * alpha * 256,
		0.0f, 0.0f, 0.0f,
		color);

	// Happy Lucky Dance Party
	fnt->setColor(0.85f, 0.85f, 0.85f);
	fnt->setSize(30.0f);
	fnt->setAlpha(alpha);
	fnt->smearDrawCentered(320.0f, 365.0f, 128.0f, _("Mobile Tactical Dancing Action"));

	fnt->setSize(36.0f);
	fnt->setAlpha(alpha * (0.5f + (fsin(frame * 2*M_PI/hz)/2)));
	fnt->smearDrawCentered(320.0f, 422.0f, 128.0f, _("Press Start!"));

	pvr_scene_finish();
}

// Do the title screen; returns false if it timed out         
//   or true if the user hit START.
Title::Result Title::doTitle() {
	// Load textures

	txr_logo = res.getTexture(Res::Title, "title", true);

	scrolly = new ScrollyBG(res.getTexture(Res::Preload, "scrolly_logo"));
	scrolly->setTranslate(Vector(0,0,10));

	// Load sounds
	RefPtr<Sound> sfx_announce = res.getSound(Res::Title, "FeetOfFury_super_echo");
	RefPtr<Sound> sfx_accept = res.getSound(Res::Preload, "title_accept");

	fnt = res.getFont("axaxax");

	oggVolume(1.0f);
	oggStart("bgm/title.ogg", true, false, false);

	timer_ms_gettime(&start_s, &start_ms);

	frame = 0;
	alpha = 0.0f;
	bool done = false, quitting = false, timedout = false;
	int hitstart;
	while (!done) {
		hitstart = checkCont();
		if (hitstart == -1)
			return Reset;
			
		if (!quitting && hitstart == 1)
			sfx_accept->play();
		if (hitstart == 2)
			timedout = true;
		quitting = quitting || hitstart;
		
		drawFrame();

		if (!quitting && alpha < 1.0f) {
			alpha += 0.05f;
			if (alpha >= 1.0f) {
				sfx_announce->play();
				alpha = 1.0f;
			}
		} else if (quitting && alpha > 0.0f)
			alpha -= 0.1f;
		else if (quitting && alpha <= 0.0f)
			done = true;
		frame++;

		if (quitting) {
			oggVolume(alpha);
		}
	}

	alpha = 0.0f;
	oggStop(false);

	drawFrame();
	drawFrame();
	drawFrame();

	// Give a bit of time for the sounds to stop gracefully
	if (!timedout) {
		printf("being graceful\n");
		thd_sleep(500);
	}
	
	fnt = NULL;
	scrolly = NULL;
	
	txr_logo = NULL;

	snd_sfx_stop_all();

	return timedout ? Timeout : Start;
}

